
/**
 * goIRC (C)opyright 2007 Sebastian Kasparek
 *
 * ircparser.cpp - parse the irc protocol
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ircparser.h"

class IrcParser::MyPrivate
{
public:
	os::RegExp *m_pcRegExpOuter;
	os::RegExp *m_pcRegExpPrefix;
	os::RegExp *m_pcRegExpParams;
};

IrcParser::IrcParser()
{
	my = new MyPrivate();
	my->m_pcRegExpOuter = new os::RegExp();
	// XXX: polish regexp for user and host (maybe sub sub expressions possible?)
	my->m_pcRegExpPrefix = new os::RegExp();
//      my->m_pcRegExpParams = new os::RegExp( "^([ ]+[^ ]*)*([ ]+:.*)?$",
//              false, true
//      );
}

IrcMessage IrcParser::ParseLine( os::String cLine )
{
	IrcMessage cIrcMessage;

	// split 3 mainparts of the irc line
	if( my->m_pcRegExpOuter->Compile( "^(:[^ ]*)?[ ]*([0-9]{3}|[a-zA-Z]*)([ ]*.*)$", false, true ) == EOK )
	{
		if( !my->m_pcRegExpOuter->Search( cLine ) )
		{
			std::cout << "os::RegExp.Compile(Outer): " << cLine.c_str() << " << is not a valid irc line?" << std::endl;
			return cIrcMessage;
		}
		else
		{
			// seperate prefix, command and parameters
			std::vector < os::String >cVectorOuter = my->m_pcRegExpOuter->GetSubStrList();
			cIrcMessage.m_cPrefix = cVectorOuter.at( 0 );
			cIrcMessage.m_cCommand = cVectorOuter.at( 1 ).Upper();
			cIrcMessage.m_cParams = cVectorOuter.at( 2 );
			// parse prefix
			if( !cIrcMessage.m_cPrefix.empty() )
			{
				if( my->m_pcRegExpPrefix->Compile( "^:([]a-zA-Z0-9\\.[`^{}_|-]*)(![^@]*)?(@[a-zA-Z0-9.-]*)?$", false, true ) == EOK )
				{
					if( !my->m_pcRegExpPrefix->Search( cIrcMessage.m_cPrefix ) )
					{
						std::cout << "os::RegExp.Search(Prefix): " << cIrcMessage.m_cPrefix.c_str() << " << not a valid irc prefix?" << std::endl;
						return cIrcMessage;
					}
					else
					{
						std::vector < os::String >cVectorPrefix = my->m_pcRegExpPrefix->GetSubStrList();
						cIrcMessage.m_cSender = cVectorPrefix.at( 0 );
						cIrcMessage.m_cUser = cVectorPrefix.at( 1 );
						if( !cIrcMessage.m_cUser.empty() )
							cIrcMessage.m_cUser = cIrcMessage.m_cUser.substr( 1 );
						cIrcMessage.m_cHost = cVectorPrefix.at( 2 );
						if( !cIrcMessage.m_cHost.empty() )
							cIrcMessage.m_cHost = cIrcMessage.m_cHost.substr( 1 );
					}
				}
				else
				{
					std::cout << "os::RegExp.Compile(Prefix): " << cIrcMessage.m_cPrefix.c_str() << " << is not a valid irc prefix?" << std::endl;
					return cIrcMessage;
				}
			}
			else
			{
				std::cout << "PREFIX: prefix string is an empty line?" << std::endl;
				return cIrcMessage;
			}

			// TODO: translate command

			// seperate Parameters
			if( !cIrcMessage.m_cParams.empty() )
			{
				if( cIrcMessage.m_cParams.substr( 1, 1 ) == ":" )
				{
					cIrcMessage.m_cVectorParams.push_back( cIrcMessage.m_cParams.substr( 2 ) );
				}
				else
				{
					unsigned int nParamsLength = cIrcMessage.m_cParams.Length();
					os::String cParam = "";
					for( unsigned int i = 0; i < nParamsLength; i++ )
					{
						if( i < ( nParamsLength - 2 ) && cIrcMessage.m_cParams[i] == ' ' && cIrcMessage.m_cParams[i + 1] == ':' )
						{
							if( !cParam.empty() )
								cIrcMessage.m_cVectorParams.push_back( cParam );
							cIrcMessage.m_cVectorParams.push_back( cIrcMessage.m_cParams.substr( i + 2 ) );
							i = nParamsLength;
						}
						else if( cIrcMessage.m_cParams[i] == ' ' )
						{
							if( !cParam.empty() )
								cIrcMessage.m_cVectorParams.push_back( cParam );
							cParam = "";
						}
						else if( i < ( nParamsLength - 1 ) )
						{
							cParam += cIrcMessage.m_cParams[i];
						}
						else if( i == ( nParamsLength - 1 ) )
						{
							cParam += cIrcMessage.m_cParams[i];
							cIrcMessage.m_cVectorParams.push_back( cParam );
						}
					}
				}
			}
		}
	}
	else
	{
		std::cout << "os::RegExp.Compile(Outer): " << cLine.c_str() << " << not a valid irc line?" << std::endl;
	}
	return cIrcMessage;
}

IrcParser::~IrcParser()
{
	delete( my->m_pcRegExpOuter );
	delete( my->m_pcRegExpPrefix );
//	delete( my->m_pcRegExpParams );
	delete( my );
}
