/**
 * goIRC (C)opyright 2007 Sebastian Kasparek
 *
 * main.cpp - commandline parser and Application initialization
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "application.h"
#include "defines.h"

int main( int argc, char *argv[] )
{
	// unbuffered output for debug messages on stdout
	setbuf( stdout, NULL );

	os::OptionParser * pcOpts = new os::OptionParser( argc, argv );
	pcOpts->AddArgMap( "nick", 'n', "nickname to use" );
	pcOpts->AddArgMap( "server", 's', "servername or ip" );
	pcOpts->AddArgMap( "port", 'p', "port to connect the server" );
	pcOpts->AddArgMap( "password", 'P', "server password" );
	pcOpts->AddArgMap( "encoding", 'e', "default encoding converted with iconv to UTF-8" );
	pcOpts->AddArgMap( "version", 'v', "version of this application" );
	pcOpts->AddArgMap( "help", 'h', "this help message" );
	pcOpts->ParseOptions( "nspPevh" );

	if( pcOpts->FindOption( 'h' ) != 0 )
	{
		std::cout << "goIRC " << GOIRC_VERSION << std::endl << "usage:" << std::endl << pcOpts->GetHelpText().str(  ) << std::endl;
		std::cout << "Please take the long form for options that have a value." << std::endl;
		std::cout << "The short form does not support values." << std::endl;
		std::cout << "E.g. '-s irc.freenode.org' will not work!" << std::endl;
		std::cout << "Use '--server=irc.freenode.org' instead." << std::endl << std::endl;
		return ( 0 );
	}

	if( pcOpts->FindOption( 'v' ) != 0 )
	{
		std::cout << "version: goIRC " << GOIRC_VERSION << std::endl;
		return ( 0 );
	}

	App *pcApp = new App( pcOpts );
	pcApp->Run();

	return ( 0 );
}
