
/**
 * goIRC (C)opyright 2007 Sebastian Kasparek
 *
 * mainwindow.cpp - view component in the mvc like architecture
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "mainwindow.h"
#include "defines.h"
#include "messages.h"
#include "textinputview.h"
#include "settingsdialog.h"

class MainWindow::MyPrivate
{
      public:
	void DisplayString( os::String cString );
	os::LayoutView * m_pcView;
	os::VLayoutNode * m_pcRoot;
	os::Menu * m_pcMenuBar;
	os::Menu * m_pcMenuApplication;
	os::TextView * m_pcTextViewTopic;
	os::TextView * m_pcTextView;
	os::HLayoutNode * m_pcHLayoutBottom;
	os::StringView * m_pcStringViewNick;
	os::StringView * m_pcStringViewChannel;
	TextInputView *m_pcTextInputView;

	os::Messenger m_cMessengerController;
};

void MainWindow::MyPrivate::DisplayString( os::String cString )
{
	// add new text at the end of the TextView buffer
	os::TextView::buffer_type cBuffer = m_pcTextView->GetBuffer();

	m_pcTextView->Insert( os::IPoint( cBuffer[cBuffer.size() - 1].size(), cBuffer.size() - 1 ), cString.c_str() );
}

MainWindow::MainWindow():os::Window( os::Rect( 0, 0, 600, 400 ), "goirc_main_wnd", "goIRC - Good Old Internet Relay Chat" )
{
	my = new MyPrivate();

	// get the default fixed width font
	os::font_properties sFontProperties;
	os::Font::GetDefaultFont( DEFAULT_FONT_FIXED, &sFontProperties );
	os::Font *pcFont = new os::Font( sFontProperties );

	// fill the mainwindow with the widgets
	my->m_pcView = new os::LayoutView( GetBounds(), "layout_view" );
	my->m_pcRoot = new os::VLayoutNode( "Root" );

	my->m_pcMenuBar = new os::Menu( os::Rect( 0, 0, 0, 0 ), "MenuBar", os::ITEMS_IN_ROW );
	my->m_pcMenuBar->SetFrame( os::Rect( 0, 0, GetBounds().Width() + 1, my->m_pcMenuBar->GetPreferredSize( false ).y + 1 ) );
	my->m_pcMenuApplication = new os::Menu( os::Rect( 0, 0, 0, 0 ), "Application", os::ITEMS_IN_COLUMN );
	my->m_pcMenuApplication->AddItem( "About...", new os::Message( M_APP_ABOUT ) );
	my->m_pcMenuApplication->AddItem( new os::MenuSeparator() );
	my->m_pcMenuApplication->AddItem( "Settings...", new os::Message( M_APP_SETTINGS ) );
	my->m_pcMenuApplication->AddItem( new os::MenuSeparator() );
	my->m_pcMenuApplication->AddItem( "Quit", new os::Message( M_APP_QUIT ), "CTRL+Q" );
	// FIXME: the shortcut doesn't seem to be added automatically by the menu?
	AddShortcut( os::ShortcutKey( "CTRL+Q" ), new os::Message( M_APP_QUIT ) );
	my->m_pcMenuBar->AddItem( my->m_pcMenuApplication );
	my->m_pcMenuBar->SetTargetForItems( this );
	my->m_pcRoot->AddChild( my->m_pcMenuBar );

	my->m_pcTextViewTopic = new os::TextView( os::Rect(), "TextViewTopic", "No topic is set", os::CF_FOLLOW_ALL, os::WID_WILL_DRAW );
	my->m_pcTextViewTopic->SetReadOnly( true );
	my->m_pcTextViewTopic->SetFont( pcFont );
	my->m_pcRoot->AddChild( my->m_pcTextViewTopic, 1.0 );
	my->m_pcTextView = new os::TextView( os::Rect(), "TextView", "", os::CF_FOLLOW_ALL, os::WID_WILL_DRAW );
	my->m_pcTextView->SetMultiLine( true );
	my->m_pcTextView->SetReadOnly( true );
	my->m_pcTextView->SetFont( pcFont );
	// my->m_pcTextView->MakeCsrVisible(); FIXME: doesn't do anything? I did expect it would make the cursor visible
	my->m_pcRoot->AddChild( my->m_pcTextView, 1.0 );

	my->m_pcHLayoutBottom = new os::HLayoutNode( "HLayoutBottom", 0.00001 );
	my->m_pcStringViewChannel = new os::StringView( os::Rect(), "StringViewCannel", "---, ", os::ALIGN_LEFT, os::CF_FOLLOW_ALL, os::WID_WILL_DRAW );
	my->m_pcStringViewChannel->SetFont( pcFont );
	my->m_pcHLayoutBottom->AddChild( my->m_pcStringViewChannel, 0.00001 );

	my->m_pcStringViewNick = new os::StringView( os::Rect(), "StringViewNick", "Nick", os::ALIGN_LEFT, os::CF_FOLLOW_ALL, os::WID_WILL_DRAW );
	my->m_pcStringViewNick->SetFont( pcFont );
	my->m_pcHLayoutBottom->AddChild( my->m_pcStringViewNick, 0.00001 );

	my->m_pcTextInputView = new TextInputView( os::Rect(), "TextInputView", "", os::Application::GetInstance() );
	my->m_pcTextInputView->SetFont( pcFont );
	my->m_pcHLayoutBottom->AddChild( my->m_pcTextInputView, 1.0 );
	my->m_pcRoot->AddChild( my->m_pcHLayoutBottom );

	my->m_pcView->SetRoot( my->m_pcRoot );
	AddChild( my->m_pcView );

	// set the icon
	os::Resources cCol( get_image_id() );
	os::ResStream *pcStream = cCol.GetResourceStream( "icon48x48.png" );
	os::BitmapImage *pcIcon = new os::BitmapImage( pcStream );
	SetIcon( pcIcon->LockBitmap() );
	delete( pcIcon );

	// initialize the Controller Messenger
	my->m_cMessengerController = os::Messenger( os::Application::GetInstance() );
}

MainWindow::~MainWindow()
{
	delete( my );
}

void MainWindow::HandleMessage( os::Message * pcMessage )
{
	switch ( pcMessage->GetCode() )
	{
		case M_WND_DISPLAY:
		{
			os::String cString = "";
			pcMessage->FindString( "display", &cString );
			my->DisplayString( cString );
			break;
		}
		case M_WND_CLEAR:
		{
			my->m_pcTextView->SetValue( os::String ( "" ) );
			break;
		}
		case M_WND_CHANGENICK:
		{
			os::String cString = "";
			pcMessage->FindString( "nick", &cString );
			my->m_pcStringViewNick->SetString( cString + " " );
			my->m_pcView->InvalidateLayout();
			break;
		}
		case M_WND_CHANGECHANNEL:
		{
			os::String cString = "";
			pcMessage->FindString( "channel", &cString );
			my->m_pcStringViewChannel->SetString( cString + " " );
			pcMessage->FindString( "topic", &cString );
			my->m_pcTextViewTopic->SetValue( cString );
			my->m_pcView->InvalidateLayout();
			break;
		}
		case M_WND_CHANGETOPIC:
		{
			os::String cString = "";
			pcMessage->FindString( "topic", &cString );
			my->m_pcTextViewTopic->SetValue( cString );
			break;
		}
		case M_APP_ABOUT:
		{
			os::Alert * pcAbout = new os::Alert( "About goIRC",
				os::String ( "goIRC " ) + GOIRC_VERSION + "\nGood Old Internet Relay Chat\n\n"
				"A simple IRC Client for Syllable\n\n"
				"Written by Sebastian \"kasbah\" Kasparek, 2007\n\n"
				"goIRC is released under the Gnu Public Licencse (GPL)\n"
				"Please see the file COPYING, distributed with goIRC,\n"
				"for more information",
				os::Alert::ALERT_INFO, 0x00, "OK", NULL );

			pcAbout->Go( NULL );
			break;
		}
		case M_APP_SETTINGS:
		{
			// create the settings dialog
			SettingsDialog *pcSettingsDialog = new SettingsDialog();
			pcSettingsDialog->CenterInScreen();
			pcSettingsDialog->Show();
			pcSettingsDialog->MakeFocus();
			break;
		}
		case M_APP_QUIT:
		{
			PostMessage( os::M_QUIT );
			break;
		}
		default:
		{
			std::cout << "not handled Message in MainWindow: " << pcMessage->GetCode() << std::endl;
			Handler::HandleMessage( pcMessage );
			break;
		}
	}
}

bool MainWindow::OkToQuit()
{
	// just quit, the controller will take care of closing connections
	my->m_cMessengerController.SendMessage( M_APP_QUIT );
	return ( true );
}
