/**
 * goIRC (C)opyright 2007 Sebastian Kasparek
 *
 * settingsdialog.cpp - settings dialog combining mvc in one class
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "settingsdialog.h"

class SettingsDialog::MyPrivate
{
public:
	os::LayoutView * m_pcView;
	os::VLayoutNode * m_pcRoot;
	os::HLayoutNode * m_pcHLayoutServer;
	os::StringView * m_pcStringViewServer;
	os::HLayoutSpacer * m_pcHLayoutSpacerServer;
	os::TextView * m_pcTextViewServer;
	os::HLayoutNode * m_pcHLayoutNodePort;
	os::StringView * m_pcStringViewPort;
	os::HLayoutSpacer * m_pcHLayoutSpacerPort;
	os::Spinner * m_pcSpinnerPort;
	os::HLayoutNode * m_pcHLayoutNodeNick;
	os::StringView * m_pcStringViewNick;
	os::HLayoutSpacer * m_pcHLayoutSpacerNick;
	os::TextView * m_pcTextViewNick;
	os::HLayoutNode * m_pcHLayoutNodePassword;
	os::StringView * m_pcStringViewPassword;
	os::HLayoutSpacer * m_pcHLayoutSpacerPassword;
	os::TextView * m_pcTextViewPassword;
	os::HLayoutNode * m_pcHLayoutNodeEncoding;
	os::StringView * m_pcStringViewEncoding;
	os::HLayoutSpacer * m_pcHLayoutSpacerEncoding;
	os::DropdownMenu * m_pcDropdownMenuEncoding;
	os::HLayoutSpacer * m_pcHLayoutSpacerPostDropdown;
	os::VLayoutSpacer * m_pcVLayoutSpacerButtons;
	os::HLayoutNode * m_pcHLayoutNodeSeparator;
	os::Separator * m_pcSeparator;
	os::HLayoutNode * m_pcHLayoutNodeButtons;
	os::HLayoutSpacer * m_pcHLayoutSpacerPreButtons;
	os::Button * m_pcButtonOk;
	os::HLayoutSpacer * m_pcHLayoutSpacerButtons;
	os::Button * m_pcButtonCancel;
};

SettingsDialog::SettingsDialog() : os::Window( os::Rect( 0, 0, 320, 220 ), "settings_wnd", "Server - Settings" )
{
	my = new MyPrivate();

	os::Settings cSettings;
	cSettings.Load();

	my->m_pcView = new os::LayoutView( GetBounds(), "layout_view" );
	my->m_pcRoot = new os::VLayoutNode( "Root" );

	my->m_pcHLayoutServer = new os::HLayoutNode( "HLayoutServer", 0.1 );
	my->m_pcHLayoutServer->SetBorders( os::Rect( 10.0, 10.0, 10.0, 10.0 ) );
	my->m_pcRoot->AddChild( my->m_pcHLayoutServer );
	my->m_pcStringViewServer = new os::StringView( os::Rect(), "StringViewServer", "Server: ", os::ALIGN_RIGHT );
	my->m_pcHLayoutServer->AddChild( my->m_pcStringViewServer, 0.1 );
	my->m_pcHLayoutSpacerServer = new os::HLayoutSpacer( "HLayoutSpacerServer", 5.0, 5.0, NULL, 0.1 );
	my->m_pcHLayoutServer->AddChild( my->m_pcHLayoutSpacerServer );
	my->m_pcTextViewServer = new os::TextView( os::Rect(), "TextViewServer", cSettings.GetString( "server" ).c_str() );
	my->m_pcTextViewServer->SetTabOrder( 0 );
	my->m_pcHLayoutServer->AddChild( my->m_pcTextViewServer, 1.0 );

	my->m_pcHLayoutNodePort = new os::HLayoutNode( "HLayoutNodePort", 0.1 );
	my->m_pcHLayoutNodePort->SetBorders( os::Rect( 10.0, 0.0, 10.0, 10.0 ) );
	my->m_pcRoot->AddChild( my->m_pcHLayoutNodePort );
	my->m_pcStringViewPort = new os::StringView( os::Rect(), "StringViewPort", "Port: ", os::ALIGN_RIGHT );
	my->m_pcHLayoutNodePort->AddChild( my->m_pcStringViewPort, 0.1 );
	my->m_pcHLayoutSpacerPort = new os::HLayoutSpacer( "HLayoutSpacerPort", 5.0, 5.0, NULL, 0.1 );
	my->m_pcHLayoutNodePort->AddChild( my->m_pcHLayoutSpacerPort );
	my->m_pcSpinnerPort = new os::Spinner( os::Rect(), "SpinnerPort", cSettings.GetInt16( "port", 6667 ), NULL );
	my->m_pcSpinnerPort->SetMinMax( 0.0, 65535.0 );
	my->m_pcSpinnerPort->SetStep( 1.0 );
	my->m_pcSpinnerPort->SetFormat( "%5.f" );
	my->m_pcSpinnerPort->SetTabOrder( 1 );
	my->m_pcHLayoutNodePort->AddChild( my->m_pcSpinnerPort, 1.0 );

	my->m_pcHLayoutNodeNick = new os::HLayoutNode( "HLayoutNodeNick", 0.1 );
	my->m_pcHLayoutNodeNick->SetBorders( os::Rect( 10.0, 0.0, 10.0, 10.0 ) );
	my->m_pcRoot->AddChild( my->m_pcHLayoutNodeNick );
	my->m_pcStringViewNick = new os::StringView( os::Rect(), "StringViewNick", "Nick: ", os::ALIGN_RIGHT );
	my->m_pcHLayoutNodeNick->AddChild( my->m_pcStringViewNick, 0.1 );
	my->m_pcHLayoutSpacerNick = new os::HLayoutSpacer( "HLayoutSpacerNick", 5.0, 5.0, NULL, 0.1 );
	my->m_pcHLayoutNodeNick->AddChild( my->m_pcHLayoutSpacerNick );
	my->m_pcTextViewNick = new os::TextView( os::Rect(), "TextViewNick", cSettings.GetString( "nick" ).c_str() );
	my->m_pcTextViewNick->SetTabOrder( 2 );
	my->m_pcHLayoutNodeNick->AddChild( my->m_pcTextViewNick, 1.0 );

	my->m_pcHLayoutNodePassword = new os::HLayoutNode( "HLayoutNodePassword", 0.1 );
	my->m_pcHLayoutNodePassword->SetBorders( os::Rect( 10.0, 0.0, 10.0, 10.0 ) );
	my->m_pcRoot->AddChild( my->m_pcHLayoutNodePassword );
	my->m_pcStringViewPassword = new os::StringView( os::Rect(), "StringViewPassword", "Password: ", os::ALIGN_RIGHT );
	my->m_pcHLayoutNodePassword->AddChild( my->m_pcStringViewPassword, 0.1 );
	my->m_pcHLayoutSpacerPassword = new os::HLayoutSpacer( "HLayoutSpacerPassword", 5.0, 5.0, NULL, 1.0 );
	my->m_pcHLayoutNodePassword->AddChild( my->m_pcHLayoutSpacerPassword );
	my->m_pcTextViewPassword = new os::TextView( os::Rect(), "TextViewPassword", cSettings.GetString( "password" ).c_str() );
	my->m_pcTextViewPassword->SetTabOrder( 3 );
	my->m_pcHLayoutNodePassword->AddChild( my->m_pcTextViewPassword, 1.0 );

	my->m_pcHLayoutNodeEncoding = new os::HLayoutNode( "HLayoutNodeEncoding", 0.1 );
	my->m_pcHLayoutNodeEncoding->SetBorders( os::Rect( 10.0, 0.0, 10.0, 10.0 ) );
	my->m_pcRoot->AddChild( my->m_pcHLayoutNodeEncoding );
	my->m_pcStringViewEncoding = new os::StringView( os::Rect(), "StringViewEncoding", "Encoding: ", os::ALIGN_RIGHT );
	my->m_pcHLayoutNodeEncoding->AddChild( my->m_pcStringViewEncoding, 0.1 );
	my->m_pcHLayoutSpacerEncoding = new os::HLayoutSpacer( "HLayoutSpacerEncoding", 5.0, 5.0, NULL, 1.0 );
	my->m_pcHLayoutNodeEncoding->AddChild( my->m_pcHLayoutSpacerEncoding );
	my->m_pcDropdownMenuEncoding = new os::DropdownMenu( os::Rect(), "DropdownMenuEncoding" );
	my->m_pcDropdownMenuEncoding->AppendItem( "UTF-8" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-16" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-15" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-14" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-13" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-10" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-9" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-7" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-5" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-4" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-3" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-2" );
	my->m_pcDropdownMenuEncoding->AppendItem( "ISO-8859-1" );
	my->m_pcDropdownMenuEncoding->SetTabOrder( 4 );
	my->m_pcDropdownMenuEncoding->SetCurrentString( cSettings.GetString( "encoding" ) );
	my->m_pcHLayoutNodeEncoding->AddChild( my->m_pcDropdownMenuEncoding, 1.0 );
	my->m_pcHLayoutSpacerPostDropdown = new os::HLayoutSpacer( "HLayoutSpacerPostDropdown", 0.0, 5000.0, NULL, 1.0 );
	my->m_pcHLayoutNodeEncoding->AddChild( my->m_pcHLayoutSpacerPostDropdown );

	my->m_pcVLayoutSpacerButtons = new os::VLayoutSpacer( "VLayoutSpacerButtons", 5.0, 5000.0, NULL, 1.0 );
	my->m_pcRoot->AddChild( my->m_pcVLayoutSpacerButtons );
	my->m_pcHLayoutNodeSeparator = new os::HLayoutNode( "HLayoutNodeSeparator", 0.1 );
	my->m_pcHLayoutNodeSeparator->SetBorders( os::Rect( 10.0, 0.0, 10.0, 10.0 ) );
	my->m_pcRoot->AddChild( my->m_pcHLayoutNodeSeparator );
	my->m_pcSeparator = new os::Separator( os::Rect(), "Separator", os::HORIZONTAL );
	my->m_pcHLayoutNodeSeparator->AddChild( my->m_pcSeparator, 1.0 );
	my->m_pcHLayoutNodeButtons = new os::HLayoutNode( "HLayoutNodeButtons", 0.1 );
	my->m_pcHLayoutNodeButtons->SetBorders( os::Rect( 10.0, 0.0, 10.0, 10.0 ) );
	my->m_pcRoot->AddChild( my->m_pcHLayoutNodeButtons );
	my->m_pcHLayoutSpacerPreButtons = new os::HLayoutSpacer( "HLayoutSpacerPreButtons", 0.0, 5000.0, NULL, 1.0 );
	my->m_pcHLayoutNodeButtons->AddChild( my->m_pcHLayoutSpacerPreButtons );
	my->m_pcButtonOk = new os::Button( os::Rect(), "ButtonOk", "Ok", new os::Message( 1 ) );
	my->m_pcButtonOk->SetTabOrder( 5 );
	my->m_pcHLayoutNodeButtons->AddChild( my->m_pcButtonOk, 0.0 );
	my->m_pcHLayoutSpacerButtons = new os::HLayoutSpacer( "HLayoutSpacerButtons", 5.0, 5.0, NULL, 0.1 );
	my->m_pcHLayoutNodeButtons->AddChild( my->m_pcHLayoutSpacerButtons );
	my->m_pcButtonCancel = new os::Button( os::Rect(), "ButtonCancel", "Cancel", new os::Message( os::M_QUIT ) );
	my->m_pcButtonCancel->SetTabOrder( 6 );
	my->m_pcHLayoutNodeButtons->AddChild( my->m_pcButtonCancel, 0.0 );

	my->m_pcRoot->SameWidth( "StringViewServer", "StringViewPort", "StringViewNick", "StringViewPassword", "StringViewEncoding", NULL );
	my->m_pcView->SetRoot( my->m_pcRoot );
	AddChild( my->m_pcView );

	// set the icon
	os::Resources cCol( get_image_id() );
	os::ResStream * pcStream = cCol.GetResourceStream( "icon48x48.png" );
	os::BitmapImage * pcIcon = new os::BitmapImage( pcStream );
	SetIcon( pcIcon->LockBitmap() );
	delete( pcIcon );
}

void SettingsDialog::HandleMessage( os::Message * pcMessage )
{
	switch ( pcMessage->GetCode() )
	{
		case 1:
		{
			os::Settings cSettings = os::Settings();
			cSettings.SetString( "server", my->m_pcTextViewServer->GetValue().AsString() );
			cSettings.SetInt16( "port", my->m_pcSpinnerPort->GetValue().AsInt16() );
			cSettings.SetString( "nick", my->m_pcTextViewNick->GetValue().AsString() );
			cSettings.SetString( "password", my->m_pcTextViewPassword->GetValue().AsString() );
			cSettings.SetString( "encoding", my->m_pcDropdownMenuEncoding->GetCurrentString() );
			cSettings.Save();
			PostMessage( os::M_QUIT );
			break;
		}
		default:
		{
			std::cout << "not handled Message in SettingsDialog: " << pcMessage->GetCode() << std::endl;
			Handler::HandleMessage( pcMessage );
			break;
		}
	}
}

bool SettingsDialog::OkToQuit()
{
	// FIXME: add warning when there are unsaved changes
	return ( true );
}
