/**
 * goIRC (C)opyright 2007 Sebastian Kasparek
 *
 * textinputview.cpp - TextView extended to be usable as input line
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "textinputview.h"
#include "messages.h"

class TextInputView::MyPrivate
{
public:
	os::Messenger m_cMessenger;
};

TextInputView::TextInputView( const os::Rect & cLayoutBounds, char *pzTitle, char *pzBuffer, os::Messenger cMessenger ) : TextView( cLayoutBounds, pzTitle, pzBuffer )
{
	my = new MyPrivate();
	my->m_cMessenger = cMessenger;
}

// filter keys and do some action when a shortcut was pressed
void TextInputView::KeyDown( const char *pzString, const char *pzRawString, const uint32 nQualifiers )
{
	// enter pressed means send the value of the input field
	if( *pzRawString == '\n' )
	{
		os::Message cMessage( M_APP_SEND );
		cMessage.AddString( "send", this->GetValue().AsString() );
		my->m_cMessenger.SendMessage( &cMessage );
		this->Clear();
		return;
	}

	// temporarly save modifier Keys
	bool bShift = nQualifiers & os::QUAL_SHIFT;
	bool bCtrl = nQualifiers & os::QUAL_CTRL;
	bool bAlt = nQualifiers & os::QUAL_ALT;
	//bool bRepeat = nQualifiers & os::QUAL_REPEAT;
	//bool bDead = nQualifiers & os::QUAL_DEADKEY;

	// no ctrl and no alt pressed, give it to the textview
	if( !bCtrl && !bAlt )
	{
		TextView::KeyDown( pzString, pzRawString, nQualifiers );
		return;
	}

	// ctrl pressed, send the Shortcut Messages, if one is associated
	// XXX: create one general shortcut structure?
	if( bCtrl && !bAlt && !bShift )
	{
		if( strlen( pzRawString ) == 1 )
		{
			switch ( *pzRawString )
			{
				case 'q':
				case 'Q':
					my->m_cMessenger.SendMessage( M_APP_QUIT );
					break;
				// send ctrl+c, ctrl+v, ctrl+x to the TextView to let it do the copy'n'paste stuff
				case 'v':
				case 'V':
				case 'c':
				case 'C':
				case 'x':
				case 'X':
					TextView::KeyDown( pzString, pzRawString, nQualifiers );
					break;
			}
		}
	}
}
